.box-center {
    top: 50%;
    left: 50%;
    position: absolute;
    transform: translateX(-50%) translateY(-50%);
}

#messages-layout {
    width: 100%;
    bottom: 20px;
    position: absolute;
}

#messages-container {
    max-height: 82vh;
    overflow-y: scroll;
}

/* Handle default block states */
div[id ^= state] { display: none; }
div#state-connect { display: block; }

/* Handle message container scaling */
div#state-messages {
    width: 50%;
    height: 100%;
}
@media only screen and (max-width: 1000px) {
    div#state-messages {
        width: 70%;
    }
}
@media only screen and (max-width: 800px) {
    div#state-messages {
        width: 80%;
    }
}
@media only screen and (max-width: 600px) {
    div#state-messages {
        width: 90%;
    }
}
